var fs = require('fs');
const util = require('util');

function gitLog(remote, fn) {
    var exec = require('child_process').exec;
    exec(`git log --pretty='format:{"hash":"%h","commit": "%s"},' ${remote}`, (err, stdout, stderr) => {
        if (err || stderr) {
            fn(' ');
        } else {
            fn(stdout);
        }
    });
}

var beautfy = function (type, arr, fn) {
    var other = [],
        fix = [],
        feature = [],
        rebuild = [],
        msg = (type == 'simple') ? '' : '';

    for (var i = 0; i < arr.length; i++) {
        if (arr[i].commit.indexOf('rebuild:') >= 0) {
            var hash = (type == 'simple') ? arr[i].hash : '<b><u>' + arr[i].hash + '</u></b>';
            rebuild.push(hash + ' : ' + arr[i].commit.replace('rebuild:', ''));
        }
        if (arr[i].commit.indexOf('fix:') >= 0) {
            var hash = (type == 'simple') ? arr[i].hash : '<b><u>' + arr[i].hash + '</u></b>';
            fix.push(hash + ' : ' + arr[i].commit.replace('fix:', ''));
        }
        if (arr[i].commit.indexOf('feature:') >= 0) {
            var hash = (type == 'simple') ? arr[i].hash : '<b><u>' + arr[i].hash + '</u></b>';
            feature.push(hash + ' : ' + arr[i].commit.replace('feature:', ''));
        }

        if (arr[i].commit.indexOf('other:') >= 0) {
            var hash = (type == 'simple') ? arr[i].hash : '<b><u>' + arr[i].hash + '</u></b>';
            other.push(hash + ' : ' + arr[i].commit.replace('other:', ''));
        }
    }

    //(rebuild.length > 0) ? msg += '## Rebuild \n': '';
    msg += (rebuild.length > 0) ? (type == 'simple') ? '\n## Rebuild \n' : '<br/><h4><b>## Rebuild </b></h4><br/>' : '';
    rebuild.forEach(function (value, index) {
        msg += (type == 'simple') ? value + '\n' : value + '<br/>';
        //msg += value + '\n';
    });

    //(fix.length > 0) ? msg += '\n## Fix issues \n': '';
    msg += (fix.length > 0) ? (type == 'simple') ? '\n## Fix issues \n' : '<br/><h4><b>## Fix issues </b></h4><br/>' : '';
    fix.forEach(function (value, index) {
        msg += (type == 'simple') ? value + '\n' : value + '<br/>';
        //msg += value + '\n';
    });

    //(feature.length > 0) ? msg += '\n## Feature \n': '';
    msg += (feature.length > 0) ? (type == 'simple') ? '\n## Feature \n' : '<br/><h4><b>## Feature </b></h4><br/>' : '';
    feature.forEach(function (value) {
        msg += (type == 'simple') ? value + '\n' : value + '<br/>';
        //msg += value + '\n';
    });

    //(other.length > 0) ? msg += '\n## Others Modifications \n': '';
    msg += (other.length > 0) ? (type == 'simple') ? '\n## Others Modifications \n' : '<br/><h4><b>## Others Modifications </b></h4><br/>' : '';
    other.forEach(function (value) {
        msg += (type == 'simple') ? value + '\n' : value + '<br/>';
        //msg += value + '\n';
    });

    console.log(`rebuild: ${rebuild.length} fix: ${fix.length} feature: ${feature.length} other: ${other.length}`);
    fn((msg.length) ? msg : '');
}

var getVersion = function () {
    try {
        var package = require(process.cwd() + '/package.json');
        return package.version;
    } catch (ex) {
        console.error(ex);
        return {};
    }
}

var createWriteFile = function (text, type) {
    try {
        fs.writeFile('./CHANGELOG.' + type, text, function (err) {
            if (err) throw err;
            console.log('Saved CHANGELOG.' + type + '!');
        });
    } catch (ex) {
        console.log(ex);
    }
}

var main = function (remote, type, ver, file) {
    var date = new Date();
    version = getVersion();
    gitLog(remote, function (msg) {
        beautfy(type, JSON.parse('[' + msg.substring(0, msg.length - 1) + ']'), function (msg) {
            var v = (type == 'simple') ? '##' + version + ' ( ' + date.getDate() + '/' + date.getMonth() + '/' + date.getFullYear() + ' ) \n' : '<h4>##' + version + ' ( ' + date.getDate() + '/' + date.getMonth() + '/' + date.getFullYear() + ' )</h4><br/>';
            createWriteFile((ver == true) ? v : '' + msg, file);
        });
    });

}

exports.main = main;